@@ -106,6 +106,7 @@ METADATA_LV_SIZE_MB = 128
 MASTER_LV_SIZE_MB = 1024
 
 BlockSDVol = namedtuple("BlockSDVol", "name, image, parent")
+LVTags = namedtuple("LVTags", "mdslot, image, parent")
 
 log = logging.getLogger("storage.BlockSD")
 
@@ -203,27 +204,11 @@ def lvmTagDecode(s):
 
 
 def _getVolsTree(sdUUID):
-    lvs = lvm.getLV(sdUUID)
     vols = {}
-    for lv in lvs:
-        if lv.name in SPECIAL_LVS_V4:
-            # Special lvs are not user lvs.
-            continue
-
-        if sc.TAG_VOL_UNINIT in lv.tags:
-            # Uninitialized LVs have no image or parent yet.
-            continue
-
-        image = ""
-        parent = ""
-        for tag in lv.tags:
-            if tag.startswith(sc.TAG_PREFIX_IMAGE):
-                image = tag[len(sc.TAG_PREFIX_IMAGE):]
-            elif tag.startswith(sc.TAG_PREFIX_PARENT):
-                parent = tag[len(sc.TAG_PREFIX_PARENT):]
-            if parent and image:
-                vols[lv.name] = BlockSDVol(lv.name, image, parent)
-                break
+    for lv in _iter_volumes(sdUUID):
+        lvtags = parse_lv_tags(lv)
+        if lvtags.parent and lvtags.image:
+            vols[lv.name] = BlockSDVol(lv.name, lvtags.image, lvtags.parent)
         else:
             log.warning(
                 "Ignoring volume %s that lacks minimal tag set: %s",
@@ -232,6 +217,39 @@ def _getVolsTree(sdUUID):
     return vols
 
 
+def _iter_volumes(sdUUID):
+    for lv in lvm.getLV(sdUUID):
+        if lv.name in SPECIAL_LVS_V4:
+            # Exclude special volumes.
+            continue
+
+        if sc.TAG_VOL_UNINIT in lv.tags:
+            # Uninitialized LVs have no mapping yet.
+            continue
+
+        yield lv
+
+
+def parse_lv_tags(lv):
+    image = None
+    parent = None
+    mdslot = None
+
+    for tag in lv.tags:
+        if tag.startswith(sc.TAG_PREFIX_IMAGE):
+            image = tag[len(sc.TAG_PREFIX_IMAGE):]
+        elif tag.startswith(sc.TAG_PREFIX_PARENT):
+            parent = tag[len(sc.TAG_PREFIX_PARENT):]
+        elif tag.startswith(sc.TAG_PREFIX_MD):
+            try:
+                mdslot = int(tag[len(sc.TAG_PREFIX_MD):])
+            except ValueError:
+                log.warning("Invalid tag %r for lv %s/%s",
+                            tag, lv.vg_name, lv.name)
+
+    return LVTags(mdslot, image, parent)
+
+
 def getAllVolumes(sdUUID):
     """
     Return dict {volUUID: ((imgUUIDs,), parentUUID)} of the domain.
@@ -783,15 +801,8 @@ class BlockStorageDomainManifest(sd.StorageDomainManifest):
     def occupied_metadata_slots(self):
         stripPrefix = lambda s, pfx: s[len(pfx):]
         occupiedSlots = []
-        special_lvs = self.special_volumes(self.getVersion())
-        for lv in lvm.getLV(self.sdUUID):
-            if lv.name in special_lvs:
-                # Special LVs have no mapping
-                continue
 
-            if sc.TAG_VOL_UNINIT in lv.tags:
-                # Uninitialized LVs have no mapping yet.
-                continue
+        for lv in _iter_volumes(self.sdUUID):
 
             offset = None
             for tag in lv.tags:
@@ -1687,6 +1698,11 @@ class BlockStorageDomain(sd.StorageDomain):
             f.flush()
             os.fsync(f.fileno())
 
+    # Dump metadata
+
+    def dump(self):
+        return {"metadata": self.getInfo()}
+
 
 def _external_leases_path(sdUUID):
     return lvm.lvPath(sdUUID, sd.XLEASES)
